@@ -11,9 +11,6 @@
  *       notice, this list of conditions and the following disclaimer in the
  *       documentation and/or other materials provided with the distribution.
  * 
- *   3.  The name of the author may not be used to endorse or promote products
- *       derived from this software without specific prior written permission.
- * 
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
  * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MER-
  * CHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
@@ -25,8 +22,19 @@
  * ERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
+ * Alternatively, the contents of this file may be used under the terms of
+ * the GNU General Public License ("GPL") version 2 or any later version,
+ * in which case the provisions of the GPL are applicable instead of
+ * the above. If you wish to allow the use of your version of this file
+ * only under the terms of the GPL and not to allow others to use your
+ * version of this file under the BSD license, indicate your decision
+ * by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL. If you do not delete the
+ * provisions above, a recipient may use your version of this file under
+ * either the BSD or the GPL.
+ *
  * This library is modelled strictly after Ralf S. Engelschalls article at
- * http://www.gnu.org/software/pth/rse-pmt.ps.  So most of the credit must
+ * http://www.gnu.org/software/pth/rse-pmt.ps. So most of the credit must
  * go to Ralf S. Engelschall <rse@engelschall.com>.
  *
  * This coroutine library is very much stripped down. You should either
@@ -44,6 +52,19 @@
  * 2007-05-02 Add assembly versions for x86 and amd64 (to avoid reliance
  *            on SIGUSR2 and sigaltstack in Crossfire).
  * 2008-01-21 Disable CFI usage on anything but GNU/Linux.
+ * 2008-03-02 Switched to 2-clause BSD license with GPL exception.
+ * 2008-04-04 New (but highly unrecommended) pthreads backend.
+ * 2008-04-24 Reinstate CORO_LOSER (had wrong stack adjustments).
+ * 2008-10-30 Support assembly method on x86 with and without frame pointer.
+ * 2008-11-03 Use a global asm statement for CORO_ASM, idea by pippijn.
+ * 2008-11-05 Hopefully fix misaligned stacks with CORO_ASM/SETJMP.
+ * 2008-11-07 rbp wasn't saved in CORO_ASM on x86_64.
+ *            introduce coro_destroy, which is a nop except for pthreads.
+ *            speed up CORO_PTHREAD. Do no longer leak threads either.
+ *            coro_create now allows one to create source coro_contexts.
+ *            do not rely on makecontext passing a void * correctly.
+ *            try harder to get _setjmp/_longjmp.
+ *            major code cleanup/restructuring.
  */
 
 #ifndef CORO_H
@@ -68,21 +89,19 @@
  *
  *    This flavour uses SUSv2's get/set/swap/makecontext functions that
  *    unfortunately only newer unices support.
- *    Use this for GNU/Linux + glibc-2.2.3 and possibly higher.
  *
  * -DCORO_SJLJ
  *
  *    This flavour uses SUSv2's setjmp/longjmp and sigaltstack functions to
  *    do it's job. Coroutine creation is much slower than UCONTEXT, but
  *    context switching is often a bit cheaper. It should work on almost
- *    all unices. Use this for GNU/Linux + glibc-2.2. glibc-2.1 and below
- *    do not work with any sane model (neither sigaltstack nor context
- *    functions are implemented)
+ *    all unices.
  *
  * -DCORO_LINUX
  *
- *    Old GNU/Linux systems (<= glibc-2.1) work with this implementation
- *    (it is very fast and therefore recommended over other methods).
+ *    Old GNU/Linux systems (<= glibc-2.1) only work with this implementation
+ *    (it is very fast and therefore recommended over other methods, but
+ *    doesn't work with anything newer).
  *
  * -DCORO_LOSER
  *
@@ -99,8 +118,12 @@
  *    Handcoded assembly, known to work only on a few architectures/ABI:
  *    ELF Linux x86 && amd64 when gcc is used and optimisation is turned on.
  *
+ * -DCORO_PTHREAD
+ *
+ *    Use the pthread API. You have to provide <pthread.h> and -lpthread.
+ *
  * If you define neither of these symbols, coro.h will try to autodetect
- * the model.  This currently works for CORO_LOSER only. For the other
+ * the model. This currently works for CORO_LOSER only. For the other
  * alternatives you should check (e.g. using autoconf) and define the
  * following symbols: HAVE_UCONTEXT_H / HAVE_SETJMP_H / HAVE_SIGALTSTACK.
  */
@@ -122,8 +145,14 @@ typedef struct coro_context coro_context;
  * uninitialised coro_context, it expects a pointer to the entry function
  * and the single pointer value that is given to it as argument.
  *
- * Allocating/deallocating the stack is your own responsibility, so there is
- * no coro_destroy function.
+ * Allocating/deallocating the stack is your own responsibility.
+ *
+ * As a special case, if coro, arg, sptr and ssize are all zero,
+ * then an "empty" coro_contetx will be created that is suitable
+ * as an initial source for coro_transfer.
+ *
+ * This function is not reentrant, but putting a mutex around it
+ * will work.
  */
 void coro_create (coro_context *ctx, /* an uninitialised coro_context */
                   coro_func coro,    /* the coroutine code to be executed */
@@ -135,8 +164,22 @@ void coro_create (coro_context *ctx, /* an uninitialised coro_context */
  * The following prototype defines the coroutine switching function. It is
  * usually implemented as a macro, so watch out.
  *
-void coro_transfer(coro_context *prev, coro_context *next);
+ * This function is thread-safe and reentrant.
  */
+#if 0
+void coro_transfer (coro_context *prev, coro_context *next);
+#endif
+
+/*
+ * The following prototype defines the coroutine destroy function. It is
+ * usually implemented as a macro, so watch out. It also serves
+ * no purpose unless you want to use the CORO_PTHREAD backend.
+ *
+ * This function is thread-safe and reentrant.
+ */
+#if 0
+void coro_destroy (coro_context *ctx);
+#endif
 
 /*
  * That was it. No other user-visible functions are implemented here.
@@ -146,7 +189,8 @@ void coro_transfer(coro_context *prev, coro_context *next);
 
 #if !defined(CORO_LOSER) && !defined(CORO_UCONTEXT) \
     && !defined(CORO_SJLJ) && !defined(CORO_LINUX) \
-    && !defined(CORO_IRIX) && !defined(CORO_ASM)
+    && !defined(CORO_IRIX) && !defined(CORO_ASM) \
+    && !defined(CORO_PTHREAD)
 # if defined(WINDOWS)
 #  define CORO_LOSER 1 /* you don't win with windoze */
 # elif defined(__linux) && (defined(__x86) || defined (__amd64))
@@ -171,6 +215,7 @@ struct coro_context {
 };
 
 # define coro_transfer(p,n) swapcontext (&((p)->uc), &((n)->uc))
+# define coro_destroy(ctx) (void *)(ctx)
 
 #elif CORO_SJLJ || CORO_LOSER || CORO_LINUX || CORO_IRIX
 
@@ -178,26 +223,60 @@ struct coro_context {
 #  define _GNU_SOURCE /* for linux libc */
 # endif
 
+# if !CORO_LOSER
+#  include <unistd.h>
+# endif
+
+/* solaris is hopelessly borked, it expands _XOPEN_UNIX to nothing */
+# if __sun
+#  undef _XOPEN_UNIX
+#  define _XOPEN_UNIX 1
+# endif
+
 # include <setjmp.h>
 
 struct coro_context {
+#if _XOPEN_UNIX > 0 || CORO_LOSER
   jmp_buf env;
+#else
+  sigjmp_buf env;
+#endif
 };
 
-# if CORO_LINUX || (_XOPEN_SOURCE >= 600)
-#  define coro_transfer(p,n) do { if (!_setjmp ((p)->env)) _longjmp ((n)->env, 1); } while (0)
+# if _XOPEN_UNIX > 0
+#  define coro_transfer(p,n) do { if (!  _setjmp ((p)->env   ))   _longjmp ((n)->env, 1); } while (0)
+# elif CORO_LOSER
+#  define coro_transfer(p,n) do { if (!   setjmp ((p)->env   ))    longjmp ((n)->env, 1); } while (0)
 # else
-#  define coro_transfer(p,n) do { if (!setjmp  ((p)->env)) longjmp  ((n)->env, 1); } while (0)
+#  define coro_transfer(p,n) do { if (!sigsetjmp ((p)->env, 0)) siglongjmp ((n)->env, 1); } while (0)
 # endif
 
+# define coro_destroy(ctx) (void *)(ctx)
+
 #elif CORO_ASM
 
 struct coro_context {
-  volatile void **sp;
+  void **sp; /* must be at offset 0 */
 };
 
-void __attribute__ ((__noinline__, __fastcall__))
-     coro_transfer(coro_context *prev, coro_context *next);
+void __attribute__ ((__noinline__, __regparm__(2)))
+coro_transfer (coro_context *prev, coro_context *next);
+
+# define coro_destroy(ctx) (void *)(ctx)
+
+#elif CORO_PTHREAD
+
+# include <pthread.h>
+
+extern pthread_mutex_t coro_mutex;
+
+struct coro_context {
+  pthread_cond_t cv;
+  pthread_t id;
+};
+
+void coro_transfer (coro_context *prev, coro_context *next);
+void coro_destroy (coro_context *ctx);
 
 #endif
 
